/*
    San Angeles Observation - Series 60 port
    Copyright 2005, 2006 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Martin Storsj
    martin@martin.st
*/

#include <coemain.h>
#include "angelesappview.h"
#include "angelesappui.h"
#include <flogger.h>
#include "app.h"
#include <hal.h>
#include <eiklabel.h>
#include <eikbtgpc.h>
#include <angeles.rsg>
#include <avkon.rsg>
#include <stdlib.h>

extern "C" void DebugPrint(int line) {
	RFileLogger::WriteFormat(_L("debug"), _L("angeles.txt"), EFileLoggingModeAppend, _L("%d"), line);
}

extern "C" void DebugPrint2(int line, const char *str) {
	TBuf<40> buf2;
	while (*str)
		buf2.Append(*str++);
	RFileLogger::WriteFormat(_L("debug"), _L("angeles.txt"), EFileLoggingModeAppend, _L("%d %S"), line, &buf2);
}

CAngelesAppView* CAngelesAppView::NewL(const TRect& aRect) {
	CAngelesAppView* self = CAngelesAppView::NewLC(aRect);
	CleanupStack::Pop(self);
	return self;
}

CAngelesAppView* CAngelesAppView::NewLC(const TRect& aRect) {
	CAngelesAppView* self = new(ELeave) CAngelesAppView();
	CleanupStack::PushL(self);
	self->ConstructL(aRect);
	return self;
}

void CAngelesAppView::ConstructL(const TRect& aRect) {
	iState = EInited;

	CreateWindowL();

	iLabel = new(ELeave) CEikLabel();
	iLabel->SetContainerWindowL(*this);
	iLabel->SetAlignment(EHCenterVCenter);
	iLabel->SetTextL(_L("Please wait while\nprecalculating.\nThis may take\nseveral minutes."));

	SetRect(aRect);
	ActivateL();


	if (HAL::Get(HALData::ESystemTickPeriod, iPeriod) != KErrNone)
		iPeriod = 1000/64;
	else
		iPeriod /= 1000;

	iPeriodic = CPeriodic::NewL(CActive::EPriorityIdle);

	// skip the precalc/run view
	iAngeles = appInit();
	iState = ECalced;
	Run();
}


void CAngelesAppView::InitGL() {
	SetExtentToWholeScreen();

	EGLConfig config;

	iDisplay = eglGetDisplay(EGL_DEFAULT_DISPLAY);
	if (!iDisplay) {
		_LIT(KGetDisplayFailed, "eglGetDisplay failed");
		User::Panic(KGetDisplayFailed, 0);
	}

	if (eglInitialize(iDisplay, NULL, NULL) == EGL_FALSE) {
		_LIT(KInitializeFailed, "eglInitialize failed");
		User::Panic(KInitializeFailed, 0);
	}

	EGLConfig *configList = NULL;
	EGLint numConfigs = 0, configSize = 0;

	if (eglGetConfigs(iDisplay, configList, configSize, &numConfigs) == EGL_FALSE) {
		_LIT(KGetConfigsFailed, "eglGetConfigs failed");
		User::Panic(KGetConfigsFailed, 0);
	}

	configSize = numConfigs;
	configList = (EGLConfig*) User::Alloc(sizeof(EGLConfig)*configSize);
	if (!configList) {
		_LIT(KConfigAllocFailed, "config alloc failed");
		User::Panic(KConfigAllocFailed, 0);
	}

	TDisplayMode mode = Window().DisplayMode();
	TInt bufferSize = 0;

	switch (mode) {
		case EColor4K: bufferSize = 12; break;
		case EColor64K: bufferSize = 16; break;
		case EColor16M: bufferSize = 24; break;
		case EColor16MU: bufferSize = 32; break;
		case EColor16MA: bufferSize = 32; break;
		default:
			_LIT(KModeError, "unsupported displaymode");
			User::Panic(KModeError, 0);
			break;
	}

	const EGLint attribList[] = {
		EGL_BUFFER_SIZE, bufferSize, EGL_DEPTH_SIZE, 16,
		EGL_NONE
	};

	if (eglChooseConfig(iDisplay, attribList, configList, configSize, &numConfigs) == EGL_FALSE) {
		_LIT(KChooseConfigFailed, "eglChooseConfig failed");
		User::Panic(KChooseConfigFailed, 0);
	}

	config = configList[0];
	User::Free(configList);

	iSurface = eglCreateWindowSurface(iDisplay, config, &Window(), NULL);

	if (!iSurface) {
		_LIT(KCreateWindowSurfaceFailed, "eglCreateWindowSurface failed");
		User::Panic(KCreateWindowSurfaceFailed, 0);
	}

	iContext = eglCreateContext(iDisplay, config, NULL, NULL);
	if (!iContext) {
		_LIT(KCreateContextFailed, "eglCreateContext failed");
		User::Panic(KCreateContextFailed, 0);
	}

	if (eglMakeCurrent(iDisplay, iSurface, iSurface, iContext) == EGL_FALSE) {
		_LIT(KMakeCurrentFailed, "eglMakeCurrent failed");
		User::Panic(KMakeCurrentFailed, 0);
	}

	appInitGL();
}

void CAngelesAppView::SetCbaL(TInt aResource, TBool aDraw) {
	CEikButtonGroupContainer *cba = static_cast<CAknAppUi*>(iEikonEnv->AppUi())->Cba();
	cba->SetCommandSetL(aResource);
	if (aDraw) cba->DrawNow();
}

void CAngelesAppView::PrecalcL() {
	if (iState != EInited) return;

	iState = ECalcing;
	SetCbaL(R_ANGELES_CBA_EMPTY);
	DrawNow();

	// run DoPrecalc later, in order to redraw the CBA before precalcing
	iPeriodic->Start(50, 50, TCallBack(CAngelesAppView::Callback, this));
}

void CAngelesAppView::DoPrecalcL() {
	RFileLogger::WriteFormat(_L("debug"), _L("angeles.txt"), EFileLoggingModeAppend, _L("Doprecalc"));
	// init demo
	iAngeles = appInit();
	iState = ECalced;

	SetCbaL(R_ANGELES_CBA_RUN);
	DrawNow();
	RFileLogger::WriteFormat(_L("debug"), _L("angeles.txt"), EFileLoggingModeAppend, _L("Doprecalc done"));
}

void CAngelesAppView::SizeChanged() {
	iLabel->SetRect(Rect());
}

void CAngelesAppView::HandleResourceChange(TInt aType) {
	CCoeControl::HandleResourceChange(aType);
	if (aType == KEikDynamicLayoutVariantSwitch) {
		switch (iState) {
		case EInited:
		case ECalcing:
		case ECalced:
			SetRect(static_cast<CEikAppUi*>(iEikonEnv->AppUi())->ClientRect());
			break;

		case ERunning:
			SetExtentToWholeScreen();
			break;
		}
	}
}

void CAngelesAppView::Run() {
	if (iState == ERunning) return;

	SetCbaL(R_AVKON_SOFTKEYS_BACK, EFalse);

	InitGL();
	appInitState(iAngeles);

	iPeriodic->Start(100, 100, TCallBack(CAngelesAppView::Callback, this));

	iState = ERunning;
}

CAngelesAppView::CAngelesAppView() {
}


CAngelesAppView::~CAngelesAppView() {
	delete iLabel;

	delete iPeriodic;
	if (iAngeles) appDeinit(iAngeles);

	CloseSTDLIB();

	if (iDisplay != EGL_NO_DISPLAY) {
		eglMakeCurrent(iDisplay, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
		if (iSurface != EGL_NO_SURFACE) eglDestroySurface(iDisplay, iSurface);
		if (iContext != EGL_NO_CONTEXT) eglDestroyContext(iDisplay, iContext);
		eglTerminate(iDisplay);
	}

}

void CAngelesAppView::Stop() {
	// skip the precalc/run view
	static_cast<CAngelesAppUi*>(iEikonEnv->AppUi())->DoExit();

	iPeriodic->Cancel();

	eglMakeCurrent(iDisplay, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
	eglDestroySurface(iDisplay, iSurface);
	eglDestroyContext(iDisplay, iContext);
	eglTerminate(iDisplay);
	iDisplay = EGL_NO_DISPLAY;
	iSurface = EGL_NO_SURFACE;
	iContext = EGL_NO_CONTEXT;

	SetRect(static_cast<CEikAppUi*>(iEikonEnv->AppUi())->ClientRect());

	iState = ECalced;
	SetCbaL(R_ANGELES_CBA_RUN);
	DrawNow();
}

TInt CAngelesAppView::CountComponentControls() const {
	switch (iState) {
	case ECalcing:
		return 1;

	case EInited:
	case ECalced:
	case ERunning:
	default:
		return 0;
	}
}

CCoeControl* CAngelesAppView::ComponentControl(TInt aIndex) const {
	switch (aIndex) {
	case 0:
		return iLabel;
	default:
		return NULL;
	}
}

void CAngelesAppView::Draw(const TRect& aRect) const {
	CWindowGc& gc = SystemGc();
	TRect rect = Rect();
	switch (iState) {
	case EInited:
	case ECalcing:
	case ECalced:
		gc.Clear(rect);
		break;
	case ERunning:
		break;
	}
}

TInt CAngelesAppView::Callback(TAny* aArg) {
	CAngelesAppView* view = (CAngelesAppView*) aArg;

	if (view->iState == ECalcing) {
		TRAPD(err, view->DoPrecalcL());
		view->iPeriodic->Cancel();
		return 0;
	}

	appRender(view->iAngeles, User::TickCount()*view->iPeriod, view->Rect().Width(), view->Rect().Height());
	eglSwapBuffers(view->iDisplay, view->iSurface);
	User::ResetInactivityTime();

	if (!isAlive(view->iAngeles)) {
		view->Stop();
	}

	return 0;
}

